/*
 *	Copyright (C) 2009 by Chebotarev Roman <roma@ultranet.ru>
 *
 *	This program is free software; you can redistribute it and/or modify
 *	it under the terms of the GNU General Public License as published by
 *	the Free Software Foundation; either version 2 of the License.
 *
 *	This program is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU General Public License for more details.
 *
 *	You should have received a copy of the GNU General Public License
 *	along with this program; if not, write to the Free Software
 *	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "informationwindow.h"
#include "serviceclass.h"
#include "about.h"
#include "nagiosagent.h"

const QString NagiosAgent::defaultExecDir = "/nagios/cgi-bin/";
const QString NagiosAgent::defaultHostRequest = "status.cgi?hostgroup=all&style=hostdetail&hoststatustypes=12";
const QString NagiosAgent::defaultSrvcsRequest = "status.cgi?host=all&servicestatustypes=28";
const QString NagiosAgent::defaultHttpServerPort = "80";
const QString NagiosAgent::defaultHttpsServerPort = "443";
const QString NagiosAgent::defaultAutoCloseTime = "10";
const QString NagiosAgent::defaultBasicHostsRegExp =
	"^\\s*<td .*\\s?class=['\"](status_list)['\"]\\s?.*><a href=['\"](\\S+)['\"].*>(.+)</a>";
const QString NagiosAgent::defaultBasicSrvcsRegExp =
	"^\\s*<TD .*\\s?class=['\"](status_list)['\"]\\s?.*><a href=['\"](\\S+)['\"].*>(.+)</a>";
const QString NagiosAgent::defaultExpertHostsRegExp =
	"^\\s*<td .*\\s?class=['\"](statusHOSTDOWN|statusHOSTUNREACHABLE)['\"]\\s?.*><a href=['\"](\\S+)['\"].*>(.+)</a>";
const QString NagiosAgent::defaultExpertSrvcsRegExp =
	"^\\s*<TD .*\\s?class=['\"](statusBGCRITICAL)['\"]\\s?.*><a href=['\"](\\S+)['\"].*>(.+)</a>";
const QString NagiosAgent::trayToolTipHeader = "Nagios Agent";
const QString NagiosAgent::configDirName = "NetPatch";
const QString NagiosAgent::programmName = "NagiosAgent";
const QString NagiosAgent::hostUpSnd = "HostDown.wav";
const QString NagiosAgent::hostDownSnd = "HostUp.wav";
const QString NagiosAgent::programmTranslatorName = "NagiosAgentLang_";
const QString NagiosAgent::commonTranslatorName = "qt_";
const QString NagiosAgent::hostUpIcon1 = ":/icons/icon16x16-host-up-1.png";
const QString NagiosAgent::hostUpIcon2 = ":/icons/icon16x16-host-up-2.png";
const QString NagiosAgent::hostDownIcon1 = ":/icons/icon16x16-host-down-1.png";
const QString NagiosAgent::hostDownIcon2 = ":/icons/icon16x16-host-down-2.png";
const QString NagiosAgent::normalTrayIcon = ":/icons/icon16x16.png";
const QString NagiosAgent::problemTrayIcon = ":/icons/icon16x16-problem.png";

NagiosAgent::NagiosAgent(QWidget *parent)
    : QWidget(parent,
    		Qt::WindowSystemMenuHint | Qt::WindowMinimizeButtonHint |
				Qt::WindowMaximizeButtonHint | Qt::WindowTitleHint )
{
    ui.setupUi(this);
    readConfig();
	initAppResources();
	ui.retranslateUi(this);
	applyConfig();
    //Позиционируем окно в центре экрана
	QRect screenRect = QApplication::desktop()->screenGeometry(this);
	setGeometry(screenRect.width()/2 - defaultWidth/2, screenRect.height()/2 - defaultHeight/2,
				defaultWidth, defaultHeight);

	informWinPt.rx() = 100;
    informWinPt.ry() = 100;
	httpConnectingTime = 0;

	ignoredHttpSignal = true;	//Устанавливаем в true т.к. при наличии не правильного конфига (или его отсутсвия)
								//в момент задания сокетов объекты QHttp всё равно выдают сигнал который
								//не должен быть обработан.
	authReqErrorMsgVisible = false;

	httpObj = new QHttp;

    updater = new HttpUpdater(UPDATE_SERVER, UPDATE_PORT, UPDATE_PATH, UPDATE_INFO);

    createTrayIcon();
    connectSignals();
    createActions();

    if(runningConfig.useHttps)
    {
    	httpSslSocket = new QSslSocket;
    	if(runningConfig.denyInvalidCert == false)	//Разрешено принимать сертификаты с ошибками
			httpSslSocket->setPeerVerifyMode(QSslSocket::VerifyNone);
    	httpObj->setSocket(httpSslSocket);
    }
    else
    {
    	httpSocket = new QTcpSocket;
    	httpObj->setSocket(httpSocket);
    }


    if(!isConfigOk())
    {
    	hasProblem = true;
    	trouble(tr("Wrong configuration. The program needs to be reconfigured."));
    	QMessageBox::warning(this, tr("Nagios Agent: configuration error."),
    			tr("The program is not configured correctly. Please enter the correct settings."));
    	showAndReadConfig();
    	return;
    }

    hasProblem = false;
	ignoredHttpSignal = false;	//Если конфиг правильный, то первый же сигнал от объектов QHttp должен быть обработан
	quietUpdate = true;

	updateTrayToolTip(true);
    httpGetInfo();
    timerStart();
    if(!runningConfig.disableAutoUpdate)
    {
    	ui.actionCheckUpdates->setEnabled(false);
    	updater->check();
    	timerUpdateChecker.setSingleShot(true);
    	timerUpdateChecker.setInterval(defaultUpdateCheckInterval);
		timerUpdateChecker.start();
    }
}

void NagiosAgent::checkUpdateFinished()
{
	if(updater->isErrorOccured())
	{
		if(!quietUpdate)
		{
			sysTrayBufferStr.clear();
			trayIcon.showMessage("Nagios Agent",
					tr("Can't get information about latest version: '") +
					updater->errorDescription() + "'",
					QSystemTrayIcon::Warning);
		}
	} else if(updater->binVersion() > CURRENT_VERSION)
	{
		QMessageBox::information(this, "Nagios Agent",
				"<font>" +
				tr("New version of programm is available: ") + updater->strVersion() +
				tr("<br>Current version: ") + VERSION +
				tr("<br>Click <a href='") + updater->updateUrl() + tr("'>here</a> for download.")
				+ "</font>",
				QMessageBox::Information);
	}
	else if(!quietUpdate)
		QMessageBox::information(this, "Nagios Agent", tr("You are using the latest stable version of the program."),
				QMessageBox::Information);
	quietUpdate = true;
	if(!runningConfig.disableAutoUpdate)
	{
    	timerUpdateChecker.setSingleShot(true);
    	timerUpdateChecker.setInterval(defaultUpdateCheckInterval);
		timerUpdateChecker.start();
	}
	ui.actionCheckUpdates->setEnabled(true);
}

void NagiosAgent::startFlashingTray(bool up)
{
	flashingCounter = 12;
	if(timerFlash.isActive())
		timerFlash.stop();
	stateUp = up;
	currentIcon = (up)? &hostUpIcon1 : &hostDownIcon1;
	trayIcon.setIcon(QIcon(*currentIcon));
	timerFlash.start(flashingTimeout);
}

void NagiosAgent::trayFlashed()
{
	if(--flashingCounter < 1)
	{
		timerFlash.stop();
		trayIcon.setIcon(QIcon(normalTrayIcon));
		return;
	}
	if(!currentIcon)
		return;
	if(stateUp)
		currentIcon = (currentIcon != &hostUpIcon1)? &hostUpIcon1 : &hostUpIcon2;
	else
		currentIcon = (currentIcon != &hostDownIcon1)? &hostDownIcon1 : &hostDownIcon2;
	trayIcon.setIcon(QIcon(*currentIcon));
}

void NagiosAgent::installTranslators(bool install)
{
	//Загружаем файлы переводов. Сперва пробуем загрузить из файловой системы.
	//Если не получается - пробуем загрузить из ресурсов.
	QString locale = QLocale::system().name();
	if(!programmTranslator.load((appDir.path() + "/" + programmTranslatorName) + locale))
	{
		MSG("Can't load programm translator '" + programmTranslatorName + locale +
				+ "' from filesystem. Loading from internal programm resources.");
		programmTranslator.load((":/translators/NagiosAgentLang_") + locale);
	}
	if(!commonTranslator.load((appDir.path() + "/" + commonTranslatorName) + locale))
	{
		MSG("Can't load common translator '" + commonTranslatorName + locale +
				"' from filesystem. Loading from internal programm resources.");
		commonTranslator.load((":/translators/qt_") + locale);
	}

	if(install)
	{
		qApp->installTranslator(&programmTranslator);
		qApp->installTranslator(&commonTranslator);
	}
	else
	{
		qApp->removeTranslator(&programmTranslator);
		qApp->removeTranslator(&commonTranslator);
	}
}

void NagiosAgent::initAppResources()
{
	//Проверяем наличие домашнего каталога директории, если такового нет - создаём.
#ifdef	Q_OS_WIN32
#define SETTINGS_DIR	"Application Data/"
#else
#define	SETTINGS_DIR	"."
#endif
	appDir.setPath(QDir::homePath() + "/" + SETTINGS_DIR + configDirName + "/" + programmName);
	if(!appDir.exists())
	{
		MSG("Creating application directory.");
		if(!appDir.mkpath(appDir.path()))
		{
			MSG("Can't create application directory. Using system TEMP directory as application directory");
			appDir.setPath(QDir::tempPath());
		}
	}

	//Загружаем кодеки и переводчики
	QTextCodec* codec = QTextCodec::codecForName("UTF-8");
	QTextCodec::setCodecForTr(codec);
	QTextCodec::setCodecForCStrings(codec);
	QTextCodec::setCodecForLocale(codec);
    QTextCodec::setCodecForTr(QTextCodec::codecForName("utf8"));
    installTranslators(!runningConfig.disableLocalization);

    //Проверяем наличие звуков для сигнализации событий,
    //если их нет - выгружаем из ресурсов в файловую систему потому что:
    //"Note that QSound does not support resources. This might be fixed in a future Qt version."
    //( http://doc.trolltech.com/4.5/qsound.html )
	if(! QFile::exists(appDir.path() + "/" + hostUpSnd))
		QFile::copy(":/sounds/" + hostUpSnd, appDir.path() + "/" + hostUpSnd);
	if(! QFile::exists(appDir.path() + "/" + hostDownSnd))
		QFile::copy(":/sounds/" + hostDownSnd, appDir.path() + "/" + hostDownSnd);
}

void NagiosAgent::connectSignals()
{
	connect(ui.btnAddHost, SIGNAL(clicked()), this, SLOT(addHost()));
	connect(ui.btnDelHost, SIGNAL(clicked()), this, SLOT(delHost()));
	connect(ui.btnAddService, SIGNAL(clicked()), this, SLOT(addService()));
	connect(ui.btnDelService, SIGNAL(clicked()), this, SLOT(delService()));
	connect(ui.btnSave, SIGNAL(clicked()), this, SLOT(checkAndSaveConfig()));
    connect(ui.actionQuit, SIGNAL(triggered()), qApp, SLOT(quit()));
    connect(ui.actionConfig, SIGNAL(triggered()), this, SLOT(showAndReadConfig()));
    connect(httpObj, SIGNAL(requestFinished ( int , bool )), this, SLOT(processHttpFinished(int, bool)));
    connect(httpObj, SIGNAL(sslErrors (const QList<QSslError> &)), this, SLOT(processSslErrors(const QList<QSslError>&)));
    connect(&timerRequests, SIGNAL(timeout()), this, SLOT(processTimer()));
    connect(&trayHostsMenu, SIGNAL(triggered(QAction *)), this, SLOT(hostsSrvcsMenuHandler(QAction *)));
    connect(&traySrvcsMenu, SIGNAL(triggered(QAction *)), this, SLOT(hostsSrvcsMenuHandler(QAction *)));
    connect(ui.actionCloseAll, SIGNAL(triggered()), this, SLOT(closeAllPopups()));
    connect(ui.actionAbout, SIGNAL(triggered()), this, SLOT(showAbout()));
    connect(ui.cbUseHttps, SIGNAL(toggled(bool)), this, SLOT(useHttpsClicked(bool)));
    connect(&trayIcon, SIGNAL(messageClicked()), this, SLOT(trayMessageClicked()));
    connect(&timerFlash, SIGNAL(timeout()), this, SLOT(trayFlashed()));
    connect(ui.cbUseTrayMessages, SIGNAL(toggled(bool)), this, SLOT(useTrayMsgsClicked(bool)));
    connect(updater, SIGNAL(checkUpdateFinished()), this, SLOT(checkUpdateFinished()));
    connect(ui.actionCheckUpdates, SIGNAL(triggered()), this, SLOT(checkUpdates()));
    connect(&timerUpdateChecker, SIGNAL(timeout()), this, SLOT(checkUpdatesQuiet()));
}

void NagiosAgent::checkUpdatesQuiet()
{
	ui.actionCheckUpdates->setEnabled(false);
	quietUpdate = true;
	updater->check();
}

void NagiosAgent::checkUpdates()
{
	ui.actionCheckUpdates->setEnabled(false);
	quietUpdate = false;
	updater->check();
}

void NagiosAgent::trayMessageClicked()
{
	if(runningConfig.openUrlFromTray &&
			(sysTrayBufferStr.indexOf(QRegExp("^https?://"), Qt::CaseInsensitive) > -1))
		QDesktopServices::openUrl(sysTrayBufferStr);
}

void NagiosAgent::useHttpsClicked(bool use)
{
	if(use)
		ui.edServerPort->setText(defaultHttpsServerPort);
	else
		ui.edServerPort->setText(defaultHttpServerPort);
	ui.grBoxSsl->setEnabled(use);
}

void NagiosAgent::useTrayMsgsClicked(bool use)
{
	ui.cbAutoClose->setDisabled(use);
	if(!use)
	{
		if(ui.cbAutoClose->isChecked())
		{
			ui.spinBoxClose->setEnabled(true);
			ui.lbMinutes->setEnabled(true);
		}
	}
	else
	{
		ui.spinBoxClose->setEnabled(false);
		ui.lbMinutes->setEnabled(false);
	}
}

void NagiosAgent::processSslErrors(const QList<QSslError> & sslErrorsList)
{
	if(runningConfig.denyInvalidCert == false)
		return;
	static bool errorMessageVisible = false;
	if(errorMessageVisible)
		return;
	errorMessageVisible = true;

	timerRequests.stop();
	httpObj->close();

	hasProblem = true;
	trouble(tr("SSL error ocurred. ") + tr("Monitoring is impossible."));
	QString errors;
	int num = 0;
	Q_FOREACH(QSslError error, sslErrorsList)
		errors += QString::number(++num) + ") " + error.errorString() + "\n";
	switch(QMessageBox::critical(this, tr("Nagios Agent: SSL error"),
			tr("Server certificate ") + runningConfig.serverAddress + ":" + runningConfig.serverPort +
			tr(" contains the following errors:\n") + errors +
			tr("\n\"Yes\" - terminate the execution of the program"
			"\n\"No\" - open settings. On the  tab \"Misc\", you can allow"
				"SSL certificates which contain errors."),
			QMessageBox::Yes | QMessageBox::No,
			QMessageBox::No))
	{
	case QMessageBox::Yes:
		qApp->quit();
		break;
	case QMessageBox::No:
		showAndReadConfig();
		ui.tabConfig->setCurrentIndex(miscTab);
		break;
	default:
		break;
	}
	errorMessageVisible = false;
}

void NagiosAgent::showHostUpOrDownMessage(QString hostName, InformationTypes state,
								QString hostHttpLink, QString hostStatus)
{
	if(runningConfig.useTrayMessages)
	{
		sysTrayBufferStr = hostHttpLink;
		trayIcon.showMessage( tr("Change status of ")
				+ ((state == hostUp || state == hostDown)? tr("host") : tr("service")) ,
				tr("Name: ") + hostName + "\n" +
				tr("Status: ") + hostStatus,
				(state == hostDown || state == serviceDown)? QSystemTrayIcon::Critical : QSystemTrayIcon::Information
			);
	}
	else
	{
		updateInformWinPt();
		InformationWindow *b = new InformationWindow(0, state,
				"<a href='" + hostHttpLink + "'>" + hostName +"</a>",
				hostStatus,
				runningConfig.autoCloseTime,
				informWinPt, runningConfig.popupsAlwaysStayOnTop);
		connect(this, SIGNAL(closeAllInformWin()), b, SLOT(close()));
	}
	if(!runningConfig.disableSounds)
		QSound::play(appDir.path() + "/" +
				((state == hostDown || state == serviceDown)? hostDownSnd : hostUpSnd));
}

void NagiosAgent::closeAllPopups()
{
	closeAllInformWin();
}

//Функция обновляющая координаты левого верхнего угла для следующего
//информационного окна - сдвигает вправо/вниз, либо возвращает
//значение координаты к исходному значению
void NagiosAgent::updateInformWinPt()
{
	QRect screenRect = QApplication::desktop()->screenGeometry(this);
	if(informWinPt.ry() > screenRect.height() - 220)
		informWinPt.ry() = 100;		//Стартовое значение
	else informWinPt.ry() += 15;
	if(informWinPt.rx() > screenRect.width() - 380)
		informWinPt.rx() = 100;		//Стартовое значение
	else informWinPt.rx() += 15;
}

bool NagiosAgent::isConfigOk()
{
#define CONF_ERR(msg, widget, tab) { \
		QMessageBox::warning(this, tr("Nagios Agent: configuration error"), msg); \
		ui.tabConfig->setCurrentIndex(tab); \
		widget->setFocus(); \
		return false; \
	}
#define CONF_WARN(msg)	QMessageBox::information(this, tr("Nagios Agent: warning"), msg);

	if(ui.edServerAddress->text().isEmpty())
		CONF_ERR(tr("No server address of Nagios found."), ui.edServerAddress, commonTab);
	if(ui.edServerPort->text().isEmpty())
	{
		CONF_WARN(tr("Server port is not specified and will be installed in the default."));
		ui.edServerPort->setText((ui.cbUseHttps->isChecked())? defaultHttpsServerPort : defaultHttpServerPort);
	}
	else
	{
		if(ui.edServerPort->text().toInt(0, 10) < 1 ||
				ui.edServerPort->text().toInt(0, 10) > 65535)
			CONF_ERR(tr("Invalid server port number Nagios."), ui.edServerPort, commonTab);
	}
	if(ui.cbHostsRequest->isChecked())
	{
		if(ui.edHostsRequest->text().isEmpty())
			CONF_ERR(tr("No request for the status of hosts."), ui.edHostsRequest, commonTab);
	}
	if(ui.cbServicesRequest->isChecked())
	{
		if(ui.edServicesRequest->text().isEmpty())
			CONF_ERR(tr("No request for the status of services."), ui.edServicesRequest, commonTab);
	}
	if(ui.cbUseHTTPAuth->isChecked())
	{
		if(ui.edLogin->text().isEmpty())
			CONF_ERR(tr("Not specified HTTP username."), ui.edLogin, commonTab);
		if(ui.edPassword->text().isEmpty())
			CONF_ERR(tr("Not specified HTTP password."), ui.edPassword, commonTab);
	}
	if(ui.cbHostsRegExp->isChecked())
	{
		QRegExp rx(ui.edHostsRegExp->text());
		if(!rx.isValid() || !ui.edHostsRegExp->text().length())
		{
			CONF_ERR(tr("Not a valid regular expression to analyze the status of hosts."),
					ui.edHostsRegExp, statusTab);
		}
	}
	if(ui.cbSrvcsRegExp->isChecked())
	{
		QRegExp rx(ui.edSrvcsRegExp->text());
		if(!rx.isValid() || !ui.edSrvcsRegExp->text().length())
			CONF_ERR(tr("Not a valid regular expression to analyze the status of services."),
					ui.edSrvcsRegExp, statusTab);
	}
	if(ui.rbBaseRegExpConfig->isChecked())
	{
		if(!ui.cbHostDownStat->isChecked() && !ui.cbHostUnreachStat->isChecked())
			CONF_ERR(tr("Not specified no status for the host!"), ui.frameBasicRegExpConf, statusTab);
		if(!ui.cbServiceCritStat->isChecked() && !ui.cbServiceWarnStat->isChecked() &&
				!ui.cbServiceUnknownStat->isChecked())
			CONF_ERR(tr("Not specified no status for the service!"), ui.frameBasicRegExpConf, statusTab);
	}
	if(ui.cbUseHttps->isChecked())
	{
	    if (!QSslSocket::supportsSsl())
	    	CONF_ERR("<font>" +
	    			tr("You can obtain the necessary libraries for work using HTTPS can ") +
	    			tr("<a href='http://www.openssl.org/related/binaries.html'>here</a>.") +
	    			tr("The work using HTTPS is impossible now.</font>"),
	    			ui.cbUseHttps, commonTab);
	}
#undef CONF_ERR
#undef CONF_WARN
	return true;
}

void NagiosAgent::showAbout()
{
	About *p = new About();
	p->show();
}

void NagiosAgent::timerStart()
{
	timerRequests.start(runningConfig.requestPeriod * 1000); //Передаём значение переведённое из секунд в миллисекунды
}

void NagiosAgent::processTimer()
{
	timerRequests.setInterval(runningConfig.requestPeriod * 1000); //Передаём значение переведённое из секунд в миллисекунды
	httpGetInfo();
}

void NagiosAgent::processHostsInfo()
{
	if(rawHostsInfo.isEmpty())
	{
		MSG("Empty data from hosts!");
		return;
	}
	bool foundInHostsList;
	QString port;
	QAction * tmpAction;
	QStringList newHosts;	//Временный список хранящий новую информацию о упавших хостах,
							//используется для проверки текущего списка упавших хостов с новым
							//с целью определения поднявшихся хостов.

	QStringList httpStrings = rawHostsInfo.split("\n", QString::SkipEmptyParts);	//Разделяем полученный HTML текст на список строк
	QRegExp rx(runningConfig.hostsRegExp);
	rx.setCaseSensitivity(Qt::CaseInsensitive);
	//Анализируем полученные HTTP данные, в случае совпадения с регулярным выражением
	//добавляем к trayHostsMenu новый Action в поле text - имя упавшего хоста,
	//в поле data ссылка для открытия этого хоста в браузере
    Q_FOREACH(QString str,  httpStrings)
    {
    	if(rx.indexIn(str) > -1)	//Совпадение регулярного выражения
    	{
    		//Проверяем не находится ли данный хост в списке исключений
    		foundInHostsList = false;
    		if(runningConfig.hostsList.indexOf(
    				QRegExp(rx.cap(runningConfig.hostNameRxPos), Qt::CaseInsensitive, QRegExp::FixedString))
    				> -1)
				foundInHostsList = true;
    		if(foundInHostsList && runningConfig.allExceptSpecifedHosts)
    			continue;
    		else if(!foundInHostsList && runningConfig.onlyListedHosts)
    				continue;
			//Добавляем в временный список хостов для последующей проверки на поднявшиеся хосты
			newHosts << rx.cap(runningConfig.hostNameRxPos);
    		//Проверяем не добавлен ли уже данный хост в меню
    		Q_FOREACH(QAction * action, trayHostsMenu.actions())
    		{
    			if(action == ui.actionHostsDownList)
    				continue;
    			//Если уже добавлен в меню - начинаем следующую итерацию внешнего цикла
       			if(!action->text().compare(rx.cap(runningConfig.hostNameRxPos), Qt::CaseInsensitive))
    				goto next_string;
    		}

    		//Добавляем новый пункт меню
			trayHostsMenu.removeAction(ui.actionEmpty);
			tmpAction = new QAction(this);
			tmpAction->setText(rx.cap(runningConfig.hostNameRxPos));

			//Проверка - если порт не дефолтный для выбранного протокола (80/HTTP или 443/HTTP),
			//то добавляем его в номер в ссылку для меню трея
			port.clear();
			if( (runningConfig.useHttps && (runningConfig.serverPort != defaultHttpsServerPort) )
					||
				(!runningConfig.useHttps && (runningConfig.serverPort != defaultHttpServerPort) )
				)
				port = ":" + runningConfig.serverPort;

			tmpAction->setData(runningConfig.serverAddress + port + runningConfig.execDir
					+ rx.cap(runningConfig.hostLinkRxPos));

			trayHostsMenu.addAction(tmpAction);
			if(runningConfig.showHostsStateChanged)
			{
				showHostUpOrDownMessage(rx.cap(runningConfig.hostNameRxPos), hostDown,
					((runningConfig.useHttps)? "https://" : "http://") + tmpAction->data().toString(),
					rx.cap(runningConfig.hostStatRxPos).remove("status"));
			}
			if(runningConfig.flashesInTray)
				startFlashingTray(false);
next_string:;
	    }
    }
    //Сравниваем текущий список упавших хостов (в trayHostsMenu) с новым
	//списком полученном с сервера. В случае отсутствия какого-либо сервера
	//в новом списке - удаляем его из текущего списка упавших хостов
    bool ignore = false;
	Q_FOREACH(QAction * action, trayHostsMenu.actions())
	{
		if(action == ui.actionHostsDownList)
			continue;
		if(!action->data().toBool())
			continue;
		Q_FOREACH(QString host, newHosts)
			if(!host.compare(action->text(), Qt::CaseInsensitive))
				goto next_action;
		//Проверяем - не находится-ли данный хост в списке исключкний
		if(runningConfig.allExceptSpecifedHosts)
		{
			ignore = false;
			Q_FOREACH(QString host, runningConfig.hostsList)
				if(!host.compare(action->text(), Qt::CaseInsensitive))
				{
					ignore = true;
					break;
				}
		}else
		{
			ignore = true;
			Q_FOREACH(QString host, runningConfig.hostsList)
				if(!host.compare(action->text(), Qt::CaseInsensitive))
				{
					ignore = false;
					break;
				}
		}
		if(!ignore)
		{
			if(runningConfig.showHostsStateChanged)
				showHostUpOrDownMessage(action->text(), hostUp,
						((runningConfig.useHttps)? "https://" : "http://") + action->data().toString(), "Host UP");
			if(runningConfig.flashesInTray)
				startFlashingTray(true);
		}
		trayHostsMenu.removeAction(action);
		delete action;
next_action:;
	}

    if(trayHostsMenu.actions().count() < 3)
    	trayHostsMenu.addAction(ui.actionEmpty);
    //Обновляем всплывающую подсказку для иконки в трее
    updateTrayToolTip();
}

void NagiosAgent::processHttpFinished(int id, bool error)
{
	if(ignoredHttpSignal)	//Если была произведена смена протокола, то запрос обязательно вернёт ошибку
	{						//потому просто игнорируем её и сбрасываем значение переменной
	    ignoredHttpSignal = false;
	    return;
	}
	QString objType;
	QString httpData;
	if(id == idGetHosts)
	{
		if(!runningConfig.enableHosts)
			return;
		rawHostsInfo = httpObj->readAll();
		objType = tr("hosts");
	}
	else if(id == idGetSrvcs)
	{
		if(!runningConfig.enableSrvcs)
			return;
		objType = tr("service");
		rawSrvcsInfo = httpObj->readAll();
	}
	else
		return;	//Invalid request id

	if(error)
	{
		QString errTitle;
		if(id == idGetHosts)
			tr("Nagios Agent: HTTP error status of hosts request");
		else
			tr("Nagios Agent: HTTP error status of services request");
		switch(httpObj->error())
		{
		case QHttp::HostNotFound:
		case QHttp::ConnectionRefused:
		case QHttp::UnexpectedClose:
		case QHttp::InvalidResponseHeader:
		case QHttp::WrongContentLength:
		case QHttp::ProxyAuthenticationRequiredError:
			if(!hasProblem)
			{
				hasProblem = true;
				trouble(tr("HTTP error ocurred:\n") +
						httpObj->errorString() + "\n" +
						tr("Monitoring is impossible."));
				trayIcon.showMessage(tr("Nagios Agent"),
						tr("During the execution of the query\n"
								"an error occurred:\n") +
						"'" + httpObj->errorString() + "'",
						QSystemTrayIcon::Critical);
			}
		    break;
		case QHttp::AuthenticationRequiredError:
			if(authReqErrorMsgVisible)
				return;
			authReqErrorMsgVisible = true;
			timerRequests.stop();
			hasProblem = true;
			trouble(tr("HTTP error ocurred:\n") +
					tr("Invalid password or username.\n") +
					tr("Monitoring is impossible."));
			if( QMessageBox::critical(this, errTitle,
								tr("Invalid password or username. Terminate application? "
										"Clicking \"No\" will open the program settings."),
								QMessageBox::Yes | QMessageBox::No,
								QMessageBox::No) == QMessageBox::Yes)
				qApp->exit(2);
			else
			{
				showAndReadConfig();
				ui.edLogin->setFocus();
			}
			authReqErrorMsgVisible = false;
			break;
		case QHttp::Aborted:
		    	break;
		case QHttp::UnknownError:
		default:
			if(!hasProblem)
			{
				hasProblem = true;
				trouble(tr("HTTP error ocurred:\n") +
						tr("Unidentified error.\n") +
						tr("Monitoring is impossible."));
				trayIcon.showMessage(tr("Nagios Agent: HTTP error"),
						tr("During query unidentified error occurred.\n"),
						QSystemTrayIcon::Critical);
				sysTrayBufferStr.clear();
			}
			break;
		}
		return;
	}
	int httpStatus = 0;
	httpStatus = httpObj->lastResponse().statusCode();
	if( httpStatus != httpOk)
	{
		QString errTitle;
		if(id == idGetHosts)
			tr("Nagios Agent: HTTP error status of hosts request");
		else
			tr("Nagios Agent: HTTP error status of services request");
		timerRequests.stop();
		trouble(tr("HTTP error ocurred:\n"
				"Server '") +
				runningConfig.serverAddress + tr("' return error ") + QString::number(httpStatus) + "\n" +
				tr("Monitoring is impossible."));
		switch  (
				QMessageBox::warning(this, errTitle,
				tr("Error query status: Server '") +
				runningConfig.serverAddress + tr("' return error ") + QString::number(httpStatus) + "\n" +
				tr("Terminate application? (Cancel button will open the program settings.)"),
				QMessageBox::Yes | QMessageBox::No | QMessageBox::Cancel,
				QMessageBox::Cancel))
		{
			case QMessageBox::Yes:
				qApp->quit();
				break;
			case QMessageBox::No:
				timerStart();
				break;
			case QMessageBox::Cancel:
				showAndReadConfig();
				break;
			default:
				break;
		}
		hasProblem = true;
		return;
	}
	if(hasProblem)
	{
		hasProblem = false;
		trayIcon.setIcon(QIcon(normalTrayIcon));
		trayIcon.showMessage(tr("Nagios Agent"),
				tr("Connection with the server is established."),
				QSystemTrayIcon::Information);
		sysTrayBufferStr.clear();
	}
	if(id == idGetHosts)
		processHostsInfo();
	else
		processSrvcsInfo();
}

void NagiosAgent::processSrvcsInfo()
{
	if(rawSrvcsInfo.isEmpty())
	{
		MSG("Empty data from services!");
		return;
	}
	bool foundInSrvcsList = false;
	QAction * tmpAction;
	QStringList newSrvcs;	//Временный список хранящий новую информацию о упавших хостах,
							//используется для проверки текущего списка упавших хостов с новым
							//с целью определения поднявшихся хостов.
	QString port;
	QString hostSrvc;
	QStringList httpStrings = rawSrvcsInfo.split("\n", QString::SkipEmptyParts);
	QRegExp rx(runningConfig.srvcsRegExp);	//Регулярное выражение для получения информации о сервисах
	QRegExp rxHostName(
			"host=(\\S+)[&$]"	//Регулярное выражение для получения имени хоста из ссылки на проблемный сервис
	);

	rx.setCaseSensitivity(Qt::CaseInsensitive);
	//Анализируем полученные HTTP данные, в случае совпадения с регулярным выражением
	//добавляем к traySrvcsMenu новый Action в поле text - имя проблемного сервиса,
	//в поле data ссылка для открытия этого сервиса в браузере
    Q_FOREACH(QString str,  httpStrings)
    {
    	if(rx.indexIn(str) > -1)
    	{
    		if(rxHostName.indexIn(rx.cap(runningConfig.srvLinkRxPos)) < 0)	//Извлекаем из ссылки имя хоста с проблемным сервисом
    			goto next_string;											//если не удалось извлечь - обрабатываем следующую строку
    		//Проверяем не находится ли данный хост в списке исключений
    		foundInSrvcsList = false;
			if(runningConfig.srvcsList.indexOf(
					QRegExp(rx.cap(runningConfig.srvNameRxPos), Qt::CaseInsensitive, QRegExp::FixedString))
					> -1)
				foundInSrvcsList = true;
    		if(foundInSrvcsList && runningConfig.allExceptSpecifedSrvcs)
    			continue;
    		else if(!foundInSrvcsList && runningConfig.onlyListedSrvcs)
    				continue;
			//Добавляем в временный список хостов для последующей проверки на поднявшиеся хосты
    		hostSrvc = rx.cap(runningConfig.srvNameRxPos) + ":" + rxHostName.cap(1);
			newSrvcs << hostSrvc ;
    		//Проверяем не добавлен ли уже данный хост в меню
    		Q_FOREACH(QAction * action, traySrvcsMenu.actions())
    		{
    			if(action == ui.actionSrvcsProblemsList)
    				continue;
    			if(!action->text().compare(hostSrvc, Qt::CaseInsensitive))
    				goto next_string;	//Если добавлен - переходим к следующей итерации внешнего цикла
    		}

    		//Добавляем новый пункт меню
			traySrvcsMenu.removeAction(ui.actionEmpty);
			tmpAction = new QAction(this);
			tmpAction->setText(hostSrvc);

			//Проверка - если порт не дефолтный для выбранного протокола (80/HTTP или 443/HTTP),
			//то добавляем его в номер в ссылку для меню трея
			port.clear();
			if( (runningConfig.useHttps && (runningConfig.serverPort != defaultHttpsServerPort) )
					||
				(!runningConfig.useHttps && (runningConfig.serverPort != defaultHttpServerPort) )
				)
				port = ":" + runningConfig.serverPort;

			tmpAction->setData(runningConfig.serverAddress + port + runningConfig.execDir
					+ rx.cap(runningConfig.srvLinkRxPos));

			traySrvcsMenu.addAction(tmpAction);
			if(runningConfig.showSrvcsStateChanged)
			{
				showHostUpOrDownMessage(hostSrvc, serviceDown,
					((runningConfig.useHttps)? "https://" : "http://") + tmpAction->data().toString(),
					rx.cap(runningConfig.srvStatRxPos).remove("statusBG"));
			}
			if(runningConfig.flashesInTray)
				startFlashingTray(false);
next_string:;
	    }
    }
    //Сравниваем текущий список проблемных сервисов (в traySrvcsMenu) с новым
	//списком полученном с сервера. В случае отсутствия какого-либо сервиса
	//в новом списке - удаляем его из текущего списка проблемных сервисов
    bool ignore = false;
	QString hostName;
	QString currSrvc;
	Q_FOREACH(QAction * action, traySrvcsMenu.actions())
	{
		if(action == ui.actionSrvcsProblemsList)
			continue;
		//Проверяем - есть ли сервис:хост из данного пункта меню
		//в новом списке проблемных сервисов
		if(!action->data().toBool())
			continue;
		Q_FOREACH(QString service, newSrvcs)
		{
			if(!service.compare(action->text(), Qt::CaseInsensitive))
				goto next_action;	//Если есть то переходим к обработке следующего пункта меню
		}
		//Проверяем - не находится-ли данный хост в списке исключкний
		currSrvc = action->text().split(":").first();
		hostName = action->text().split(":").last();
		if(runningConfig.allExceptSpecifedSrvcs)
		{
			ignore = false;
    		Q_FOREACH(ServiceClass service, runningConfig.srvcsList)
    			if(service.isHostFound(currSrvc, hostName))
    			{
    				ignore = true;
    				break;
    			}
		}else
		{
			ignore = true;
    		Q_FOREACH(ServiceClass service, runningConfig.srvcsList)
    			if(service.isHostFound(currSrvc, hostName))
    			{
    				ignore = false;
    				break;
    			}
		}
		if(!ignore)
		{
			if(runningConfig.showSrvcsStateChanged)
				showHostUpOrDownMessage(action->text(), serviceUp,
						((runningConfig.useHttps)? "https://" : "http://") + action->data().toString(), "Service Up");
			if(runningConfig.flashesInTray)
				startFlashingTray(true);
		}
		traySrvcsMenu.removeAction(action);
		delete action;
next_action:;
	}

    if(traySrvcsMenu.actions().count() < 3)
    	traySrvcsMenu.addAction(ui.actionEmpty);
    //Обновляем всплывающую подсказку для иконки в трее
    updateTrayToolTip();
}

void NagiosAgent::updateTrayToolTip(bool reset, QString reason)
{
	QString newToolTip = trayToolTipHeader;
	if(runningConfig.serverAddress.length())
		newToolTip += QString(" (") + ((runningConfig.useHttps)? "https://" : "http://") +
			runningConfig.serverAddress + ":" + runningConfig.serverPort + ")";

	if(reset)
	{
		newToolTip += (reason.isEmpty())?
						tr("\nExpected to obtain information...") : "\n" + reason;
		trayIcon.setToolTip(newToolTip);
		return;
	}
	if(runningConfig.enableHosts)
	{
			if((trayHostsMenu.actions().count() > 2) &&
					(trayHostsMenu.actions().last() != ui.actionEmpty))
				newToolTip += tr("\nHosts down count: ") + QString::number(trayHostsMenu.actions().count() - 2);
	}
	else
		newToolTip += tr("\nMonitoring hosts disabled");
	if(runningConfig.enableSrvcs)
	{
			if((traySrvcsMenu.actions().count() > 2) &&
					(traySrvcsMenu.actions().last() != ui.actionEmpty))
				newToolTip += tr("\nProblem services: ") + QString::number(traySrvcsMenu.actions().count() - 2);
	}
	else
		newToolTip += tr("\nMonitoring service is disabled");
	newToolTip += tr("\nData obtained: ") + QDateTime::currentDateTime().toString("hh:mm:ss dd.MM.yyyy");
	trayIcon.setToolTip(newToolTip);
}

void NagiosAgent::httpGetInfo()
{
	if(runningConfig.useHTTPAuth)
		httpObj->setUser(runningConfig.httpLogin, runningConfig.httpPassword);
	else
		httpObj->setUser("", "");
	if(httpObj->state() == QHttp::Connecting)
	{	//Если QHttp объект нахдится в состоянии Connecting на момент наступления
		//времени выполнения следующего запроса, то производим ряд проверок
		//что бы вычислить наступление таймаута.
		httpConnectingTime += runningConfig.requestPeriod;
		if(httpConnectingTime >= maxHttpConnectingTime)	//Время ожидания установления соединения
		{														//превысило максимально допустимое.
			if(!hasProblem)
			{
				hasProblem = true;
				trouble(tr("HTTP error ocurred:\n") +
						tr("Connection is timed out.\n") +
						tr("Monitoring is impossible."));
				trayIcon.showMessage(tr("Nagios Agent: HTTP error"),
						tr("Timeout while connecting to server") + "\n" +
						tr("for obtaining monitoring information."),
						QSystemTrayIcon::Critical);
				sysTrayBufferStr.clear();
			}
			httpConnectingTime = 0;
		}
	}
	else
	{	//Объект QHttp не находится в состоянии Connecting - следовательно подключение уже совершено
		//информация предыдущего запроса обработана.
		httpConnectingTime = 0;
		httpObj->setHost(runningConfig.serverAddress,
						(runningConfig.useHttps)? QHttp::ConnectionModeHttps : QHttp::ConnectionModeHttp,
						runningConfig.serverPort.toInt(0, 10));
		if(runningConfig.enableHosts)
			idGetHosts = httpObj->get(runningConfig.execDir + runningConfig.hostsRequest);
		if(runningConfig.enableSrvcs)
			idGetSrvcs = httpObj->get(runningConfig.execDir + runningConfig.servicesRequest);
	}
}

inline void NagiosAgent::trouble(QString reason)
{
	trayIcon.setIcon(QIcon(problemTrayIcon));
	updateTrayToolTip(true, reason);
}

void NagiosAgent::showAndReadConfig()
{
	//Позиционируем окно настроек в центре экрана
	if(isVisible())
	{
		this->activateWindow();
		return;
	}
	QRect screenRect = QApplication::desktop()->screenGeometry(this);
	setGeometry(screenRect.width()/2 - defaultWidth/2, screenRect.height()/2 - defaultHeight/2,
				defaultWidth, defaultHeight);
	ui.tabConfig->setCurrentIndex(commonTab);
	readConfig();
	applyConfig();
	show();
}

void NagiosAgent::checkAndSaveConfig()
{
	if(!isConfigOk())
		return;
	saveConfig();
    httpGetInfo();
	timerStart();
	hide();
}

void NagiosAgent::saveConfig()
{
	QSettings config(configDirName, programmName);

	config.setValue("ServerAddress", ui.edServerAddress->text());
	if(ui.cbUseHttps->isChecked())
		config.setValue("HTTPSPort", ui.edServerPort->text());
	else
		config.setValue("HTTPPort", ui.edServerPort->text());
	config.setValue("DenySSLCert", ui.rbDenyInvalidCert->isChecked());
	config.setValue("RequestPeriod", ui.spinBoxPeriod->value());
	config.setValue("AutoClose", ui.cbAutoClose->isChecked());
	config.setValue("UseUserDefExecDir", ui.cbExecDir->isChecked());
	config.setValue("UserDefNagiosExecDir", ui.edExecDir->text());
	config.setValue("UseUserDefHostsRequest", ui.cbHostsRequest->isChecked());
	config.setValue("UseUserDefSrvcsRequest", ui.cbServicesRequest->isChecked());
	config.setValue("UseHTTPAuth", ui.cbUseHTTPAuth->isChecked());
	config.setValue("AutoCloseTime", ui.spinBoxClose->value());
	config.setValue("Login", ui.edLogin->text());
	config.setValue("Password", ui.edPassword->text());
	config.setValue("UserDefHostsRequest", ui.edHostsRequest->text());
	config.setValue("UserDefServicesRequest", ui.edServicesRequest->text());
	config.setValue("AllExceptSpecifedHosts", ui.rbAllExceptSpecifedHosts->isChecked());
	config.setValue("OnlyListedHosts", ui.rbOnlyListedHosts->isChecked());
	config.setValue("AllExceptSpecifedSrvcs", ui.rbAllExceptSpecifedSrvcs->isChecked());
	config.setValue("OnlyListedSrvcs", ui.rbOnlyListedSrvcs->isChecked());
	config.setValue("UseTrayMessages", ui.cbUseTrayMessages->isChecked());
	config.setValue("DisableSounds", ui.cbDisableSounds->isChecked());
	config.setValue("EnableHostsMonitoring", ui.cbEnableHosts->isChecked());
	config.setValue("EnableSrvcsMonitoring", ui.cbEnableSrvcs->isChecked());
	config.setValue("ShowHostsMessages", ui.cbShowHosts->isChecked());
	config.setValue("ShowSrvcsMessages", ui.cbShowSrvcs->isChecked());
	config.setValue("UseUserDefHostsRegExp", ui.cbHostsRegExp->isChecked());
	config.setValue("HostsRegExp", ui.edHostsRegExp->text());
	config.setValue("UseUserDefSrvcsRegExp", ui.cbSrvcsRegExp->isChecked());
	config.setValue("SrvcsRegExp", ui.edSrvcsRegExp->text());
	config.setValue("HostStatRxPos", ui.edHostStatRxPos->text());
	config.setValue("HostLinkRxPos", ui.edHostLinkRxPos->text());
	config.setValue("HostNameRxPos", ui.edHostNameRxPos->text());
	config.setValue("SrvStatRxPos", ui.edSrvStatRxPos->text());
	config.setValue("SrvLinkRxPos", ui.edSrvLinkRxPos->text());
	config.setValue("SrvNameRxPos", ui.edSrvNameRxPos->text());
	config.setValue("UseHTTPS", ui.cbUseHttps->isChecked());
	config.setValue("PopupsStayOnTop", ui.cbPopupsStayOnTop->isChecked());
	config.setValue("UseBasicRegExpConfig", ui.rbBaseRegExpConfig->isChecked());
	config.setValue("UseHostDownStat", ui.cbHostDownStat->isChecked());
	config.setValue("UseHostUnreachStat",ui.cbHostUnreachStat->isChecked());
	config.setValue("UseSrvCritStat", ui.cbServiceCritStat->isChecked());
	config.setValue("UseSrvWarnStat", ui.cbServiceWarnStat->isChecked());
	config.setValue("UseSrvUnknStat", ui.cbServiceUnknownStat->isChecked());
	config.setValue("OpenUrlFromTray", ui.cbOpenUrlFromTray->isChecked());
	config.setValue("DisableLocalization", ui.cbDisableLocalization->isChecked());
	config.setValue("FlashesInTray", ui.cbFlashInTray->isChecked());
	config.setValue("DisableAutoUpdate", ui.cbDisableAutoUpdate->isChecked());

	if(ui.rbHostOnLMK->isChecked())
		config.setValue("NumMenuOnLMK", hostsMenu);
	else if(ui.rbSrvcsOnLMK->isChecked())
		config.setValue("NumMenuOnLMK", srvcsMenu);
	else
		config.setValue("NumMenuOnLMK", containerMenu);


	QString string = "";
	for(int i = 0; i < ui.listHosts->count(); ++i)
		string += ui.listHosts->item(i)->text() + "|";
	config.setValue("HostsList", string);
	string.clear();
	for(int i = 0; i < ui.listServices->count(); ++i)
		string += ui.listServices->item(i)->text() + "|";
	config.setValue("ServicesList", string);

	updateRunningConfig();
	updateTrayMenus();
	updateTrayToolTip(runningConfig.enableHosts || runningConfig.enableSrvcs,
			tr("Configuration is updated.\n") +
			tr("Waiting for obtaining new information from server."));
	httpGetInfo();
	timerStart();
}

void NagiosAgent::updateTrayMenus()
{
	if(!runningConfig.enableHosts) 	//Если отменен мониторинг хостов - удаляем пункты
	{								//меню отвечающие за хосты
		Q_FOREACH(QAction * action, trayHostsMenu.actions())
		{
			if((!action->data().toBool() || action == ui.actionSrvcsProblemsList) &&
				action != ui.actionEmpty)
				continue;
			trayHostsMenu.removeAction(action);
			if(action != ui.actionEmpty)
				delete action;
		}
		trayHostsMenu.addAction(ui.actionUnmonitored);
	}
	else
	{
		trayHostsMenu.removeAction(ui.actionUnmonitored);
		if(trayHostsMenu.actions().count() < 3)
			trayHostsMenu.addAction(ui.actionEmpty);
	}
	if(!runningConfig.enableSrvcs) 	//Если отменен мониторинг сервисов - удаляем пункты
	{								//меню отвечающие за сервисы
		Q_FOREACH(QAction * action, traySrvcsMenu.actions())
		{
			if((!action->data().toBool() || action == ui.actionSrvcsProblemsList) &&
				action != ui.actionEmpty)
				continue;
			traySrvcsMenu.removeAction(action);
			if(action != ui.actionEmpty)
				delete action;
		}
		traySrvcsMenu.addAction(ui.actionUnmonitored);
	}
	else
	{
		traySrvcsMenu.removeAction(ui.actionUnmonitored);
		if(traySrvcsMenu.actions().count() < 3)
			traySrvcsMenu.addAction(ui.actionEmpty);
	}
}

void NagiosAgent::reinitQHttp(bool httpsUsed, bool useHttps)
{
    disconnect(httpObj, SIGNAL(requestFinished (int , bool )), this, SLOT(processHttpFinished(int, bool)));
    disconnect(httpObj, SIGNAL(sslErrors (const QList<QSslError> &)), this, SLOT(processSslErrors(const QList<QSslError>&)));

    ignoredHttpSignal = true;

    timerRequests.stop();

    httpObj->close();

    delete httpObj;

    httpObj = new QHttp;

    if(httpsUsed) //Текущий протокол HTTPS
    	delete httpSslSocket;
	else	//Текущий протокол HTTP
		delete httpSocket;

    if(useHttps)
    {
    	httpSslSocket = new QSslSocket;
    	if(runningConfig.denyInvalidCert == false)	//Разрешено принимать сертификаты с ошибками
    	{
    		httpSslSocket->setPeerVerifyMode(QSslSocket::VerifyNone);
    	}
    	httpObj->setSocket(httpSslSocket);
    }
    else
    {
    	httpSocket = new QTcpSocket;
    	httpObj->setSocket(httpSocket);
    }

    connect(httpObj, SIGNAL(requestFinished ( int , bool )), this, SLOT(processHttpFinished(int, bool)));
    connect(httpObj, SIGNAL(sslErrors (const QList<QSslError> &)), this, SLOT(processSslErrors(const QList<QSslError>&)));

	timerStart();
}

void NagiosAgent::updateBasicRegExp()
{
	//Обработка хостов
	QString resultExpr = "";
	QString resultRegExp = defaultBasicHostsRegExp;
	if(ui.cbHostDownStat->isChecked())
		resultExpr = "statusHOSTDOWN";
	if(ui.cbHostUnreachStat->isChecked())
		resultExpr += QString((resultExpr.length())? "|" : "") + "statusHOSTUNREACHABLE";
	resultRegExp.replace("status_list", resultExpr);
	runningConfig.hostsRegExp = resultRegExp;
	runningConfig.hostStatRxPos = defaultHostStatRxPos;
	runningConfig.hostLinkRxPos = defaultHostLinkRxPos;
	runningConfig.hostNameRxPos = defaultHostNameRxPos;
	//обработка сервисов
	resultExpr.clear();
	resultRegExp = defaultBasicSrvcsRegExp;
	if(ui.cbServiceCritStat->isChecked())
		resultExpr = "statusBGCRITICAL";
	if(ui.cbServiceWarnStat->isChecked())
		resultExpr += QString((resultExpr.length())? "|" : "") + "statusBGWARNING";
	if(ui.cbServiceUnknownStat->isChecked())
		resultExpr += QString((resultExpr.length())? "|" : "") + "statusBGUNKNOWN";
	resultRegExp.replace("status_list", resultExpr);
	runningConfig.srvcsRegExp = resultRegExp;
	runningConfig.srvStatRxPos = defaultSrvStatRxPos;
	runningConfig.srvLinkRxPos = defaultSrvLinkRxPos;
	runningConfig.srvNameRxPos = defaultSrvNameRxPos;
}

void NagiosAgent::updateRunningConfig()
{
	runningConfig.serverAddress = ui.edServerAddress->text();
	runningConfig.serverPort = ui.edServerPort->text();
	runningConfig.requestPeriod = ui.spinBoxPeriod->value();
	runningConfig.autoClose = ui.cbAutoClose->isChecked();
	runningConfig.useUserDefExecDir = ui.cbExecDir->isChecked();
	runningConfig.useUserDefHostRequest = ui.cbHostsRequest->isChecked();
	runningConfig.useUserDefSrvcsRequest = ui.cbServicesRequest->isChecked();
	runningConfig.useHTTPAuth = ui.cbUseHTTPAuth->isChecked();
	if(runningConfig.autoClose)
		runningConfig.autoCloseTime = ui.spinBoxClose->value();
	else runningConfig.autoCloseTime = 0;
	runningConfig.httpLogin = ui.edLogin->text();
	runningConfig.httpPassword = ui.edPassword->text();
	runningConfig.allExceptSpecifedHosts = ui.rbAllExceptSpecifedHosts->isChecked();
	runningConfig.onlyListedHosts = ui.rbOnlyListedHosts->isChecked();
	runningConfig.allExceptSpecifedSrvcs = ui.rbAllExceptSpecifedSrvcs->isChecked();
	runningConfig.onlyListedSrvcs = ui.rbOnlyListedSrvcs->isChecked();
	runningConfig.useTrayMessages = ui.cbUseTrayMessages->isChecked();
	runningConfig.disableSounds = ui.cbDisableSounds->isChecked();
	runningConfig.enableHosts = ui.cbEnableHosts->isChecked();
	runningConfig.popupsAlwaysStayOnTop = ui.cbPopupsStayOnTop->isChecked();
	runningConfig.useBasicRegExpConfig = ui.rbBaseRegExpConfig->isChecked();
	runningConfig.showHostsStateChanged = ui.cbShowHosts->isChecked();
	runningConfig.showSrvcsStateChanged = ui.cbShowSrvcs->isChecked();
	runningConfig.useUserDefHostsRegExp = ui.cbHostsRegExp->isChecked();
	runningConfig.useUserDefSrvcsRegExp = ui.cbSrvcsRegExp->isChecked();
	runningConfig.openUrlFromTray = ui.cbOpenUrlFromTray->isChecked();
	runningConfig.flashesInTray = ui.cbFlashInTray->isChecked();
	runningConfig.disableAutoUpdate = ui.cbDisableAutoUpdate->isChecked();
	if(runningConfig.disableAutoUpdate)
		timerUpdateChecker.stop();
	else
	{
    	timerUpdateChecker.setSingleShot(true);
    	timerUpdateChecker.setInterval(defaultUpdateCheckInterval);
		timerUpdateChecker.start();
	}

	if(ui.rbHostOnLMK->isChecked())
		runningConfig.menuNumOnLMK = hostsMenu;
	else if(ui.rbSrvcsOnLMK->isChecked())
		runningConfig.menuNumOnLMK = srvcsMenu;
	else
		runningConfig.menuNumOnLMK = containerMenu;

	//Обработка смены политики работы с SSL сертификатами
	if(runningConfig.denyInvalidCert != ui.rbDenyInvalidCert->isChecked())
	{
		runningConfig.denyInvalidCert = ui.rbDenyInvalidCert->isChecked();
		reinitQHttp(runningConfig.useHttps, runningConfig.useHttps);
	}
	//Обработка смены протокола
	if(runningConfig.useHttps != ui.cbUseHttps->isChecked())
	{
		reinitQHttp(runningConfig.useHttps, ui.cbUseHttps->isChecked());
		runningConfig.useHttps = ui.cbUseHttps->isChecked();
	}

	if(!runningConfig.enableHosts)
		idGetHosts = 0;
	runningConfig.enableSrvcs = ui.cbEnableSrvcs->isChecked();
	if(!runningConfig.enableSrvcs)
		idGetSrvcs = 0;

	if(runningConfig.useBasicRegExpConfig)
		updateBasicRegExp();
	else
	{
		if(runningConfig.useUserDefHostsRegExp)
		{
			runningConfig.hostsRegExp = ui.edHostsRegExp->text();
			runningConfig.hostStatRxPos = ui.edHostStatRxPos->text().toInt();
			runningConfig.hostLinkRxPos = ui.edHostLinkRxPos->text().toInt();
			runningConfig.hostNameRxPos = ui.edHostNameRxPos->text().toInt();
		}
		else
		{
			runningConfig.hostsRegExp = defaultExpertHostsRegExp;
			runningConfig.hostStatRxPos = defaultHostStatRxPos;
			runningConfig.hostLinkRxPos = defaultHostLinkRxPos;
			runningConfig.hostNameRxPos = defaultHostNameRxPos;
		}
		if(runningConfig.useUserDefSrvcsRegExp)
		{
			runningConfig.srvcsRegExp = ui.edSrvcsRegExp->text();
			runningConfig.srvStatRxPos = ui.edSrvStatRxPos->text().toInt();
			runningConfig.srvLinkRxPos = ui.edSrvLinkRxPos->text().toInt();
			runningConfig.srvNameRxPos = ui.edSrvNameRxPos->text().toInt();
		}
		else
		{
			runningConfig.srvcsRegExp = defaultExpertSrvcsRegExp;
			runningConfig.srvStatRxPos = defaultSrvStatRxPos;
			runningConfig.srvLinkRxPos = defaultSrvLinkRxPos;
			runningConfig.srvNameRxPos = defaultSrvNameRxPos;
		}
	}

	if(runningConfig.useUserDefExecDir)
	{
		runningConfig.execDir = ui.edExecDir->text();
		//Добавляем лидирующий слеш абсолютного пути (/cgi-bin...) если необходим
		if(runningConfig.execDir.indexOf(QRegExp("^/")) < 0)
				runningConfig.execDir.insert(0, "/");
		//Добавляем завершающий слеш пути исполняемого каталога (...cgi-bin/) если необходим
		if(runningConfig.execDir.indexOf(QRegExp("/$")) < 0)
			runningConfig.execDir.append("/");
	}
	else runningConfig.execDir = defaultExecDir;
	if(runningConfig.useUserDefHostRequest)
		runningConfig.hostsRequest = ui.edHostsRequest->text();
	else runningConfig.hostsRequest = defaultHostRequest;
	if(runningConfig.useUserDefSrvcsRequest)
		runningConfig.servicesRequest = ui.edServicesRequest->text();
	else runningConfig.servicesRequest = defaultSrvcsRequest;

	runningConfig.hostsList.clear();
	for(int i = 0; i < ui.listHosts->count(); ++i)
		runningConfig.hostsList.append(ui.listHosts->item(i)->text());
	runningConfig.srvcsList.clear();
	for(int i = 0; i < ui.listServices->count(); ++i)
		runningConfig.srvcsList.append(ui.listServices->item(i)->text());
	if(runningConfig.disableLocalization != ui.cbDisableLocalization->isChecked())
	{
		timerRequests.stop();
		httpObj->abort();
		runningConfig.disableLocalization = ui.cbDisableLocalization->isChecked();
		installTranslators(!runningConfig.disableLocalization);
		ui.retranslateUi(this);
		readConfig();
		applyConfig();
		hasProblem = false;
		timerStart();
	}
}

void NagiosAgent::applyConfig()
{
	QSettings config(configDirName, programmName);
	ui.listHosts->clear();
	ui.listServices->clear();
	ui.cbHostDownStat->setChecked(config.value("UseHostDownStat", defaultUseHostDownStat).toBool());
	ui.cbHostUnreachStat->setChecked(config.value("UseHostUnreachStat",defaultUseHostUnreachStat).toBool());
	ui.cbServiceCritStat->setChecked(config.value("UseSrvCritStat", defaultUseSrvCritStat ).toBool());
	ui.cbServiceWarnStat->setChecked(config.value("UseSrvWarnStat", defaultUseSrvWarnStat).toBool());
	ui.cbServiceUnknownStat->setChecked(config.value("UseSrvUnknStat", defaultUseSrvUnknStat).toBool());
	ui.edServerAddress->setText(runningConfig.serverAddress);
    ui.edServerPort->setText(runningConfig.serverPort);
    ui.spinBoxPeriod->setValue(runningConfig.requestPeriod);
	ui.cbAutoClose->setChecked(runningConfig.autoClose);
	ui.cbHostsRequest->setChecked(runningConfig.useUserDefHostRequest);
	ui.cbServicesRequest->setChecked(runningConfig.useUserDefSrvcsRequest);
	ui.cbUseHTTPAuth->setChecked(runningConfig.useHTTPAuth);
    ui.spinBoxClose->setValue(config.value("AutoCloseTime", defaultAutoCloseTime).toInt());
    ui.edLogin->setText(runningConfig.httpLogin);
    ui.edPassword->setText(runningConfig.httpPassword);
    ui.cbExecDir->setChecked(runningConfig.useUserDefExecDir);
    ui.edExecDir->setText(config.value("UserDefNagiosExecDir", defaultExecDir).toString());
    ui.edHostsRequest->setText(config.value("UserDefHostsRequest", defaultHostRequest).toString());
    ui.edServicesRequest->setText(config.value("UserDefServicesRequest", defaultSrvcsRequest).toString());
    ui.rbAllExceptSpecifedHosts->setChecked(runningConfig.allExceptSpecifedHosts);
    ui.rbOnlyListedHosts->setChecked(runningConfig.onlyListedHosts);
    ui.rbAllExceptSpecifedSrvcs->setChecked(runningConfig.allExceptSpecifedSrvcs);
    ui.rbOnlyListedSrvcs->setChecked(runningConfig.onlyListedSrvcs);
    ui.cbUseTrayMessages->setChecked(runningConfig.useTrayMessages);
    ui.cbDisableSounds->setChecked(runningConfig.disableSounds);
    ui.cbEnableHosts->setChecked(runningConfig.enableHosts);
    ui.cbEnableSrvcs->setChecked(runningConfig.enableSrvcs);
    ui.cbShowHosts->setChecked(runningConfig.showHostsStateChanged);
    ui.cbShowSrvcs->setChecked(runningConfig.showSrvcsStateChanged);
    ui.cbShowHosts->setEnabled(runningConfig.enableHosts);
    ui.cbShowSrvcs->setEnabled(runningConfig.enableSrvcs);
    ui.cbHostsRegExp->setChecked(runningConfig.useUserDefHostsRegExp);
    ui.edHostsRegExp->setText(config.value("HostsRegExp", defaultExpertHostsRegExp).toString());
    ui.cbSrvcsRegExp->setChecked(runningConfig.useUserDefSrvcsRegExp);
    ui.edSrvcsRegExp->setText(config.value("SrvcsRegExp", defaultExpertSrvcsRegExp).toString());
    ui.edHostStatRxPos->setText(QString::number(runningConfig.hostStatRxPos));
	ui.edHostLinkRxPos->setText(QString::number(runningConfig.hostLinkRxPos));
	ui.edHostNameRxPos->setText(QString::number(runningConfig.hostNameRxPos));
	ui.edSrvStatRxPos->setText(QString::number(runningConfig.srvStatRxPos));
	ui.edSrvLinkRxPos->setText(QString::number(runningConfig.srvLinkRxPos));
	ui.edSrvNameRxPos->setText(QString::number(runningConfig.srvNameRxPos));
	ui.cbUseHttps->setChecked(runningConfig.useHttps);
	ui.cbPopupsStayOnTop->setChecked(runningConfig.popupsAlwaysStayOnTop);
	ui.cbOpenUrlFromTray->setChecked(runningConfig.openUrlFromTray);
	ui.cbDisableLocalization->setChecked(runningConfig.disableLocalization);
	ui.grBoxSsl->setEnabled(runningConfig.useHttps);
	ui.cbFlashInTray->setChecked(runningConfig.flashesInTray);
	ui.rbDenyInvalidCert->setChecked(runningConfig.denyInvalidCert);
	ui.rbAllowInvalidCert->setChecked(!runningConfig.denyInvalidCert);
	ui.cbDisableAutoUpdate->setChecked(runningConfig.disableAutoUpdate);

	useTrayMsgsClicked(runningConfig.useTrayMessages);

	switch(runningConfig.menuNumOnLMK)
	{
	case hostsMenu:
		ui.rbHostOnLMK->setChecked(true);
		break;
	case srvcsMenu:
		ui.rbSrvcsOnLMK->setChecked(true);
		break;
	case containerMenu:
		ui.rbHostsAndSrvcsOnLMK->setChecked(true);
		break;
	default:
		ui.rbHostOnLMK->setChecked(true);
		break;
	}
	if(runningConfig.useBasicRegExpConfig)
	{
		ui.rbBaseRegExpConfig->setChecked(true);
		ui.frameBasicRegExpConf->setEnabled(true);
		ui.frameExpertRegExpConf->setDisabled(true);
	}
	else
	{
		ui.rbExpertRegExpConfig->setChecked(true);
		ui.frameExpertRegExpConf->setEnabled(true);
		ui.frameBasicRegExpConf->setDisabled(true);
	}

	Q_FOREACH(QString s,  runningConfig.hostsList)
    	ui.listHosts->addItem(s);
    Q_FOREACH(QString s, runningConfig.srvcsList)
    	ui.listServices->addItem(s);

	if(runningConfig.useBasicRegExpConfig)
		updateBasicRegExp();
	else
	{
		if(runningConfig.useUserDefHostsRegExp)
		{
			runningConfig.hostsRegExp = config.value("HostsRegExp", defaultExpertHostsRegExp).toString();
			runningConfig.hostStatRxPos = config.value("HostStatRxPos", defaultHostStatRxPos).toInt();
			runningConfig.hostLinkRxPos = config.value("HostLinkRxPos", defaultHostLinkRxPos).toInt();
			runningConfig.hostNameRxPos = config.value("HostNameRxPos", defaultHostNameRxPos).toInt();
		}
		else
		{
			runningConfig.hostsRegExp = defaultExpertHostsRegExp;
			runningConfig.hostStatRxPos = defaultHostStatRxPos;
			runningConfig.hostLinkRxPos = defaultHostLinkRxPos;
			runningConfig.hostNameRxPos = defaultHostNameRxPos;
		}
		if(runningConfig.useUserDefSrvcsRegExp)
		{
			runningConfig.srvcsRegExp = config.value("SrvcsRegExp", defaultExpertSrvcsRegExp).toString();
			runningConfig.srvStatRxPos = config.value("SrvStatRxPos", defaultSrvStatRxPos).toInt();
			runningConfig.srvLinkRxPos = config.value("SrvLinkRxPos", defaultSrvLinkRxPos).toInt();
			runningConfig.srvNameRxPos = config.value("SrvNameRxPos", defaultSrvNameRxPos).toInt();
		}
		else
		{
			runningConfig.srvcsRegExp = defaultExpertSrvcsRegExp;
			runningConfig.srvStatRxPos = defaultSrvStatRxPos;
			runningConfig.srvLinkRxPos = defaultSrvLinkRxPos;
			runningConfig.srvNameRxPos = defaultSrvNameRxPos;
		}
	}
}

void NagiosAgent::readConfig()
{
	QSettings config(configDirName, programmName);

	runningConfig.serverAddress = config.value("ServerAddress", "").toString();
	runningConfig.useHttps = config.value("UseHTTPS", defaultUseHttps).toBool();
	if(runningConfig.useHttps)
		runningConfig.serverPort = config.value("HTTPSPort", defaultHttpsServerPort).toString();
	else
		runningConfig.serverPort = config.value("HTTPPort", defaultHttpServerPort).toString();
	runningConfig.denyInvalidCert = config.value("DenySSLCert", defaultDenyInvalidCert).toBool();
	runningConfig.requestPeriod = config.value("RequestPeriod", defaultRequestPeriod).toInt();
	runningConfig.useHTTPAuth = config.value("UseHTTPAuth", useHTTPAuth).toBool();
	runningConfig.autoClose = config.value("AutoClose", defaultAutoClose).toBool();
	if(runningConfig.autoClose)
		runningConfig.autoCloseTime = config.value("AutoCloseTime", defaultAutoCloseTime).toInt();
	else runningConfig.autoCloseTime = 0;
	runningConfig.httpLogin = config.value("Login", "").toString();
	runningConfig.httpPassword = config.value("Password", "").toString();
	runningConfig.useUserDefHostRequest = config.value("UseUserDefHostsRequest", useUserDefHostsRequest).toBool();
	runningConfig.useUserDefExecDir = config.value("UseUserDefExecDir", useUserDefExecDir).toBool();
	if(runningConfig.useUserDefExecDir)
	{
		runningConfig.execDir = config.value("UserDefNagiosExecDir", defaultExecDir).toString();
		//Добавляем лидирующий слеш абсолютного пути (/cgi-bin...) если необходим
		if(runningConfig.execDir.indexOf(QRegExp("^/")) < 0)
				runningConfig.execDir.insert(0, "/");
		//Добавляем завершающий слеш пути исполняемого каталога (...cgi-bin/) если необходим
		if(runningConfig.execDir.indexOf(QRegExp("/$")) < 0)
			runningConfig.execDir.append("/");
	}
	else runningConfig.execDir = defaultExecDir;
	if(runningConfig.useUserDefHostRequest)
		runningConfig.hostsRequest = config.value("UserDefHostsRequest", defaultHostRequest).toString();
	else runningConfig.hostsRequest = defaultHostRequest;
	runningConfig.useUserDefSrvcsRequest = config.value("UseUserDefSrvcsRequest", useUserDefSrvcsRequest).toBool();
	if(runningConfig.useUserDefSrvcsRequest)
		runningConfig.servicesRequest = config.value("UserDefServicesRequest", defaultSrvcsRequest).toString();
	else runningConfig.servicesRequest = defaultSrvcsRequest;
	runningConfig.allExceptSpecifedHosts = config.value("AllExceptSpecifedHosts", defaultAllExceptSpecifedHosts).toBool();
	runningConfig.onlyListedHosts = config.value("OnlyListedHosts", defaultOnlyListedHosts).toBool();
	runningConfig.allExceptSpecifedSrvcs = config.value("AllExceptSpecifedSrvcs", defaultAllExceptSpecifedSrvcs).toBool();
	runningConfig.onlyListedSrvcs = config.value("OnlyListedSrvcs", defaultOnlyListedSrvcs).toBool();
	runningConfig.hostsList = config.value("HostsList", "").toString().split("|", QString::SkipEmptyParts);
	runningConfig.srvcsList = config.value("ServicesList", "").toString().split("|", QString::SkipEmptyParts);
	runningConfig.useTrayMessages = config.value("UseTrayMessages", false).toBool();
	runningConfig.disableSounds = config.value("DisableSounds", false).toBool();
	runningConfig.enableHosts = config.value("EnableHostsMonitoring", true).toBool();
	runningConfig.enableSrvcs = config.value("EnableSrvcsMonitoring", true).toBool();
	runningConfig.showHostsStateChanged = config.value("ShowHostsMessages", true).toBool();
	runningConfig.showSrvcsStateChanged = config.value("ShowSrvcsMessages", true).toBool();
	runningConfig.useUserDefHostsRegExp = config.value("UseUserDefHostsRegExp", false).toBool();
	runningConfig.useUserDefSrvcsRegExp = config.value("UseUserDefSrvcsRegExp", false).toBool();
	runningConfig.popupsAlwaysStayOnTop = config.value("PopupsStayOnTop", defaultPopupsStayOnTop).toBool();
	runningConfig.useBasicRegExpConfig = config.value("UseBasicRegExpConfig", defaultUseBasicRegExpConfig).toBool();
	runningConfig.openUrlFromTray = config.value("OpenUrlFromTray", defaultOpenUrlFromTray).toBool();
	runningConfig.menuNumOnLMK = config.value("NumMenuOnLMK", defaultMenuNumOnLMK).toInt();
	runningConfig.disableLocalization = config.value("DisableLocalization", defaultDisableLocalization).toBool();
	runningConfig.flashesInTray = config.value("FlashesInTray", defaultFlashesInTray).toBool();
	runningConfig.disableAutoUpdate = config.value("DisableAutoUpdate", defaultDisableAutoUpdate).toBool();
}

void NagiosAgent::createActions()
{
    ui.actionCancel->setShortcut(Qt::Key_Escape);
    ui.btnCancel->addAction(ui.actionCancel);
}

void NagiosAgent::hostsSrvcsMenuHandler(QAction * action)
{
	//Проверка - если порт не дефолтный для выбранного протокола (80/HTTP или 443/HTTP),
	//то добавляем его в номер в ссылку для меню трея
	QString port;
	if( (runningConfig.useHttps && (runningConfig.serverPort != defaultHttpsServerPort) )
			||
		(!runningConfig.useHttps && (runningConfig.serverPort != defaultHttpServerPort) )
		)
		port = ":" + runningConfig.serverPort;

	if( (action == ui.actionHostsDownList) ||  (action == ui.actionSrvcsProblemsList))
	{
		QDesktopServices::openUrl(((runningConfig.useHttps)? "https://" : "http://") +
				runningConfig.serverAddress + port +
				runningConfig.execDir +
				( (action == ui.actionHostsDownList) ?
						runningConfig.hostsRequest : runningConfig.servicesRequest) );
	}

	if(!action->data().toBool())
		return;

	QDesktopServices::openUrl(((runningConfig.useHttps)? "https://" : "http://") + action->data().toString());
}

void NagiosAgent::createTrayIcon()
{
    //Create main menu
    trayMainMenu.addAction(ui.actionConfig);
    trayMainMenu.addSeparator();
    trayMainMenu.addAction(ui.actionCheckUpdates);
    trayMainMenu.addAction(ui.actionAbout);
    trayMainMenu.addSeparator();
    trayMainMenu.addAction(ui.actionCloseAll);
    trayMainMenu.addSeparator();
    trayMainMenu.addAction(ui.actionQuit);

    ui.actionEmpty->setDisabled(true);
    ui.actionUnmonitored->setDisabled(true);
    //Create hosts menu
    trayHostsMenu.addAction(ui.actionHostsDownList);
    trayHostsMenu.addSeparator();
    trayHostsMenu.addAction((runningConfig.enableHosts)? ui.actionEmpty : ui.actionUnmonitored);

    //Create services menu
    traySrvcsMenu.addAction(ui.actionSrvcsProblemsList);
    traySrvcsMenu.addSeparator();
    traySrvcsMenu.addAction((runningConfig.enableSrvcs)? ui.actionEmpty : ui.actionUnmonitored);

    //Создаём иконку в трее без проверки доступности трея:
    //If the system tray is currently unavailable but becomes available later,
    //QSystemTrayIcon will automatically add an entry in the system tray if it is visible.
    //Источник: http://doc.trolltech.com/4.5/qsystemtrayicon.html#isSystemTrayAvailable
	/*if (!QSystemTrayIcon::isSystemTrayAvailable())
	{
        QMessageBox::critical(this, QObject::tr("Systray"),
                              QObject::tr("I couldn't detect any system tray on this system."));
        exit(-1);
    }*/
    //Да, я знаю что плохо пытаться создать иконку в системном трее не проверив его доступность
    //т.к. может случиться что системный трей не создастся никогда, но если выполнять проверку
    //перед запуском, то в ОС отличных от Windows приложение не запускается если оно находится в автостарте
    //по прчине того что запуск приложения выполняется раньше чем запуск системного трея.

    trayIcon.setIcon(QIcon(normalTrayIcon));
	trayIcon.setToolTip("NagiosAgent");
	trayIcon.show();
	trayIcon.setContextMenu(&trayMainMenu);
	connect(&trayIcon, SIGNAL(activated(QSystemTrayIcon::ActivationReason)),
	            this, SLOT(iconActivated(QSystemTrayIcon::ActivationReason)));
}

void NagiosAgent::closeEvent(QCloseEvent * event)
{
	hide();
	event->ignore();
}

void NagiosAgent::iconActivated(QSystemTrayIcon::ActivationReason reason)
{
	QPoint pt = QCursor::pos();
	QMenu * menu = 0;
	switch (reason)
	{
	case QSystemTrayIcon::Trigger:
	{
		if(runningConfig.menuNumOnLMK == hostsMenu)
			menu = &trayHostsMenu;
		else if(runningConfig.menuNumOnLMK == srvcsMenu)
			menu = &traySrvcsMenu;
		else
		{
			trayHostsAndSrvcsMenu.clear();
			trayHostsAndSrvcsMenu.addActions(trayHostsMenu.actions());
			trayHostsAndSrvcsMenu.addSeparator();
			trayHostsAndSrvcsMenu.addActions(traySrvcsMenu.actions());
			menu = &trayHostsAndSrvcsMenu;
		}
		break;
	}
	case QSystemTrayIcon::MiddleClick:
		activateWindow();
		if(runningConfig.menuNumOnLMK == srvcsMenu)
			menu = &trayHostsMenu;
		else if(runningConfig.menuNumOnLMK == hostsMenu)
			menu = &traySrvcsMenu;
		break;
	case QSystemTrayIcon::Unknown:
	default:
		return;
	}

	if(!menu)
		return;

	activateWindow();

	if(pt.x() + menu->sizeHint().width() > QApplication::desktop()->screenGeometry(this).width())
	{
		if(pt.x() - menu->sizeHint().width() < 0)
			pt.rx() = 0;
		else
			pt.rx() -= menu->sizeHint().width();
	}

	if(pt.y() + menu->sizeHint().height() > QApplication::desktop()->screenGeometry(this).height())
		pt.ry() -= menu->sizeHint().height();

	menu->exec(pt);
}

void NagiosAgent::addService()
{
	if(!ui.edAddService->text().length())
		return;
	ui.listServices->addItem(ui.edAddService->text());
	ui.edAddService->clear();
}

void NagiosAgent::delService()
{
	qDeleteAll(ui.listServices->selectedItems());
}

void NagiosAgent::addHost()
{
	if(!ui.edAddHost->text().length())
		return;
	ui.listHosts->addItem(ui.edAddHost->text());
	ui.edAddHost->clear();
}

void NagiosAgent::delHost()
{
	qDeleteAll(ui.listHosts->selectedItems());
}

NagiosAgent::~NagiosAgent()
{
	timerRequests.stop();
	httpObj->abort();
}
